# Copyright (C) 2006 Canonical Limited.
#
# Licence to be determined
#

"""A conflict finder for packages, the main driver routine."""

import sys

from bzrlib import progress

from findconflicts import (
    ArchiveDatabase,
    ComponentGroupingHandler,
    MissingConflictFinder,
    PackageDataSource,
    PackageVersion,
    )


class Main(object):
    """The command line driver for findconflicts."""

    def __init__(self, archive_db=None):
        """Create a Main() object.

        Much of the functionality is derived from other objects which can be
        safely replaced - for instance, to allow different backends to be used.
        The attributes for these objects are:
        source: By default a findconflicts.PackageDataSource.
        progress: A progress bar such as bzrlib.progress.ProgressBar.

        :param archive_db: Specify a replacement ArchiveDatabase implementation
            to use.
        """
        self.archive_db = archive_db or ArchiveDatabase
        self.finder = MissingConflictFinder()
        self.handler = ComponentGroupingHandler('../public_html/possible-conflicts')
        self.source = PackageDataSource()
        self.store = self.archive_db.open_or_create()
        self.log = sys.stdout
        self.PackageVersion = PackageVersion
        self.progress = progress.ProgressBar()

    def run(self, argv):
        """Run the commmand with the argv argv.

        :param argv: A vector of command line arguments, such as sys.argv.
        """
        self.store.execute("PRAGMA cache_size=300000;").get_all()
        published_packages, pkg_components = self.source.get_all_packages()
        new_packages = sorted(self.PackageVersion.Find_new_versions(
            published_packages, self.store))
        total = len(new_packages)
        #try:
        for index, package in enumerate(new_packages):
            #self.progress.update(str(package), index, total)
            self.log.write("Scanning %s_%s_%s\n" % package)
            self.log.flush()
            info = self.source.infoFromPackageUrl(published_packages[package])
            self.PackageVersion.InsertFromInfo(info, self.store)
            # not strictly necessary, but nice to reduce incremental work.
            self.store.commit()
        #finally:
        #    self.progress.clear()
        # now analyze as much as possible.
        package_ids = self.PackageVersion.To_analyze(self.store, published_packages)
        self.handler.pkg_components = pkg_components
        self.PackageVersion.Analyse_ids(
            package_ids, self.store, self.finder, pkg_components, self.log, self.handler)
