# Copyright (C) 2006 Canonical Limited.
#
# Licence to be determined
#

"""Tests for the file collection which is the tool for finding conflicts."""

from unittest import TestCase

from findconflicts import (
    BinaryPackageFiles,
    )


class TestBinaryPackageFiles(TestCase):

    def test_construct(self):
        collection = BinaryPackageFiles()

    def test_add_first_files(self):
        collection = BinaryPackageFiles()
        # adding a group of files to the collection when empty just returns
        # an empty dict.
        self.assertEqual(
            {},
            collection.add_files([
                'usr/share/doc/atomix/README',
                'usr/share/doc/atomix/AUTHORS',
                ], ('a', '1.2.1', 'all')))

    def test_add_files_not_overlapping(self):
        collection = BinaryPackageFiles()
        # add some files
        collection.add_files([
            'usr/foo',
            'usr/bar',
            ], ('a', '1.2.1', 'all'))
        # now add files that dont conflict
        # we use a different arch and package because they 
        # should be irrelevant
        self.assertEqual(
            {},
            collection.add_files([
                'usr/gam',
                'usr/foo-bar',
                ], ('b', '1.2.2', 'i386')))

    def test_add_files_overlapping(self):
        collection = BinaryPackageFiles()
        # add two sets of files which dont overlap
        package_a = ('a', '1.2.1', 'all')
        package_b = ('b', '1.2.2', 'i386')
        package_c = ('c', '0', 'sparc')
        collection.add_files([
            'usr/foo',
            'usr/bar',
            ], package_a)
        collection.add_files([
            'usr/gam',
            'usr/foo-bar',
            ], package_b)
        # and add a file set that overlaps both
        # this has a different package and arch, because that should be
        # irrelevant - its filtered later on.
        self.assertEqual(
            {frozenset((package_a, package_c)):['usr/foo'],
             frozenset((package_b, package_c)):['usr/gam'],
             },
            collection.add_files([
                'usr/foo',
                'usr/gam',
                ], package_c))

    def test_add_files_overlapping_same_package_name(self):
        collection = BinaryPackageFiles()
        # add two sets of files which overlap with the same package name
        package_a = ('a', '1.2.1', 'all')
        package_b = ('a', '1.2.2', 'all')
        collection.add_files([
            'usr/foo',
            'usr/bar',
            ], package_a)
        # packages with the same name should be filtered by this 
        # function, because its the common case for file reuse and
        # we can trivially exclude the same packages files during the
        # file lookup. Not filtering this would give a very large number
        # of duplicates because every file in the same successive package
        # version will overlap.
        self.assertEqual(
            {},
            collection.add_files([
                'usr/foo',
                'usr/gam',
                ], package_b)
            )
