# Copyright (C) 2006 Canonical Limited.
#
# Licence to be determined
#

"""Tests for the getpackagesummary routines."""

from unittest import TestCase

from findconflicts import PackageDataSource

class TestGetKnownPackage(TestCase):
    
    def test_diversions_using_at(self):
        # test that analysing a package where we cannot determine
        # diversions sets the diversions field to ['UNKNOWN'], allowing 
        # later correction or detection. ['UNKNOWN'] is impossible within 
        # debian - no leading /.
        source = PackageDataSource()
        info = source.infoFromPackageUrl(
            'http://archive.ubuntu.com/ubuntu/pool/main/a/at/at_3.1.8-11ubuntu5_amd64.deb')
        self.assertEqual(['UNKNOWN'], info['diversions'])

    def test_diversions_using_slocate(self):
        # test that retrieving the slocate info returns its diversions.
        # slocate is small and diverts something. 
        # TODO: create a little package on the fly.
        source = PackageDataSource()
        info = source.infoFromPackageUrl(
            'http://archive.ubuntu.com/ubuntu/pool/main/s/slocate/slocate_3.1-1_i386.deb')
        self.assertEqual([
            '/usr/bin/locate',
            '/usr/bin/updatedb',
            '/usr/share/man/man1/locate.1.gz',
            '/usr/share/man/man1/updatedb.1.gz',
            '/etc/cron.daily/find',
            ], info['diversions'])

    def test_get_atomix_sparc(self):
        # test that we can retrieve and get the right information for
        # atomix_1.2.1-1_sparc.deb (found in
        # http://archive.ubuntu.com/ubuntu/pool/main/a/atomix/). We use this
        # package because its quite small. TODO: create a synthetic package
        # on the fly or embed one in the source (urk).
        source = PackageDataSource()
        # a package without replaces or conflicts
        info = source.infoFromPackageUrl(
            'http://archive.ubuntu.com/ubuntu/pool/main/a/atomix/atomix_1.2.1-1_sparc.deb')
        self.assertEqual({
            'controltext':'Package: atomix\nVersion: 1.2.1-1\nSection: games\nPriority: optional\nArchitecture: sparc\nDepends: libart-2.0-2 (>= 2.3.16), libatk1.0-0 (>= 1.9.0), libbonobo2-0 (>= 2.8.0), libbonoboui2-0 (>= 2.5.4), libc6 (>= 2.3.4-1), libgconf2-4 (>= 2.9), libglib2.0-0 (>= 2.7.0), libgnome2-0 (>= 2.8.0), libgnomecanvas2-0 (>= 2.11.1), libgnomeui-0 (>= 2.8.0), libgnomevfs2-0 (>= 2.10.0-0), libgtk2.0-0 (>= 2.6.0), libice6, liborbit2 (>= 1:2.10.0), libpango1.0-0 (>= 1.8.1), libpopt0 (>= 1.7), libsm6, libxml2 (>= 2.6.17), zlib1g (>= 1:1.2.1), atomix-data (= 1.2.1-1)\nInstalled-Size: 152\nMaintainer: Guilherme de S. Pastore <gpastore@colband.com.br>\nDescription: puzzle game for building molecules out of isolated atoms\n Atomix is a game designed for GNOME in which you have to build molecules,\n from simple inorganic to extremely complex organic ones, out of isolated\n atoms.\n .\n The first levels can be misleadingly easy, but added complexity always\n comes with the next level, so that several minutes (or even much more\n than that) can be spent on getting out of one single level.\n\n\n',
            'files':[
                'usr/share/doc/atomix/README',
                'usr/share/doc/atomix/AUTHORS',
                'usr/share/doc/atomix/copyright',
                'usr/share/doc/atomix/changelog.gz',
                'usr/share/doc/atomix/NEWS.gz',
                'usr/share/doc/atomix/changelog.Debian.gz',
                'usr/lib/menu/atomix',
                'usr/games/atomix'],
            'replaces':'',
            'conflicts':'',
            'name':'atomix',
            'version':'1.2.1-1',
            'arch':'sparc',
            'provides':'',
            },
            info)
        # check a package that has a replaces and conflicts clauses.
        info = source.infoFromPackageUrl(
            'http://archive.ubuntu.com/ubuntu/pool/universe/9/915resolution/915resolution_0.5-1ubuntu6_amd64.deb')
        self.assertEqual({
            'controltext':"Package: 915resolution\nVersion: 0.5-1ubuntu6\nSection: x11\nPriority: extra\nArchitecture: amd64\nDepends: libc6 (>= 2.3.4-1)\nRecommends: vbetool (>= 0.6-1)\nConflicts: 855resolution\nReplaces: 855resolution\nProvides: 855resolution\nInstalled-Size: 124\nMaintainer: Steffen Joeris <steffen.joeris@skolelinux.de>\nDescription: resolution modify tool for Intel graphic chipset\n 915resolution is a tool to modify the video BIOS of the 800\n and 900 series Intel graphics chipsets. This includes the 845G,\n 855G, and 865G chipsets, as well as 915G, 915GM, and 945G chipsets.\n This modification is necessary to allow the display of certain\n graphics resolutions for an Xorg or XFree86 graphics server.\n .\n 915resolution's modifications of the BIOS are transient.\n There is no risk of permanent modification of the BIOS.\n This also means that 915resolution must be run every time the\n computer boots inorder for it's changes to take effect.\n If you want to automatically set the resolution on each boot\n and before X is launched, see /usr/share/doc/915resolution/README.Debian\n for information about configuring the provided initscript.\n .\n Web site: http://www.geocities.com/stomljen/\n\n\n",
            'files':[
                'etc/acpi/resume.d/49-915-resolution-set.sh',
                'etc/default/915resolution',
                'etc/init.d/915resolution',
                'usr/sbin/915resolution',
                'usr/share/doc/915resolution/README.txt.gz',
                'usr/share/doc/915resolution/chipset_info.txt',
                'usr/share/doc/915resolution/README.Debian',
                'usr/share/doc/915resolution/copyright',
                'usr/share/doc/915resolution/changelog.Debian.gz',
                'usr/share/man/man8/915resolution.8.gz',
                'etc/acpi/resume.d/13-915-resolution-set.sh'
                ],
            'replaces':'855resolution',
            'conflicts':'855resolution',
            'name':'915resolution',
            'version':'0.5-1ubuntu6',
            'arch':'amd64',
            'provides':'855resolution',
            },
            info)

    def test_info_on_languagepack(self):
        # language packs have a bz2 data section - check we can parse that
        # by tring.
        source = PackageDataSource()
        info = source.infoFromPackageUrl(
            'http://archive.ubuntu.com/ubuntu/pool/main/l/language-pack-en-base/language-pack-en-base_20051011_all.deb')

    def test_get_package_list(self):
        # the PackageDataSource class offers access to the germinate
        # TagFile resource to get package lists.
        # we can get an individual package listing with just the core data
        # that we need by asking for the listing for the packages from within
        # a mirror, architecture, distribution, for a component.
        # TODO: make a sample file locally and run a local server instead of
        # talking to the internet during the test suite. This will let us
        # ensure that its doing what we think it is, as we can't test the
        # full thing with real data because real data changes.
        #
        # TODO: we might want to use the Contents files, though because
        # they lose data - the package version is stripped - we currently
        # dont. If we did though we would not need to download binaries
        # at all for processing data from the archive. 
        # 
        # we want the following data to choose what we download later:
        # package name, version, architecture, url for the pkg file to 
        # get the contents.
        source = PackageDataSource()
        packages = source.get_packages(
            mirror='http://archive.ubuntu.com/ubuntu/',
            distribution='dapper',
            component='main',
            arch='sparc')
        self.assertAtomixSparcURL(packages)
        self.assertSimpleTalSummary(packages)
        # mirror can be defined and then not supplied
        source.set_mirror('http://archive.ubuntu.com/ubuntu/')
        # change just the arch, we should get a different atomix and the 
        # same simple-tal.
        packages = source.get_packages(
            mirror='http://archive.ubuntu.com/ubuntu/',
            distribution='dapper',
            component='main',
            arch='i386')
        self.assertAtomixi386Summary(packages)
        self.assertSimpleTalSummary(packages)
        # now we try a different component and arch.
        packages = source.get_packages(
            distribution='dapper',
            component='universe',
            arch='amd64')
        self.assertMeschachSummary(packages)
        packages = source.get_packages(
            distribution='breezy',
            component='restricted',
            arch='powerpc')
        self.assertLRMSummary(packages)
        packages = source.get_packages(
            distribution='breezy',
            component='restricted',
            arch='i386')
        self.assertLRMi386Summary(packages)

    def assertLRMSummary(self, packages):
        self.assertEqual(
            'http://archive.ubuntu.com/ubuntu/pool/restricted/l/linux-meta/linux-restricted-modules-powerpc_2.6.12.16_powerpc.deb',
            packages[(
                'linux-restricted-modules-powerpc', '2.6.12.16', 'powerpc')])

    def assertLRMi386Summary(self, packages):
        self.assertEqual(
            'http://archive.ubuntu.com/ubuntu/pool/restricted/l/linux-restricted-modules-2.6.12/linux-restricted-modules-2.6.12-9-k7-nvidia-legacy_2.6.12.4-11_i386.deb',
            packages[(
                'linux-restricted-modules-2.6.12-9-k7-nvidia-legacy',
                '2.6.12.4-11',
                'i386'
                )])

    def assertMeschachSummary(self, packages):
        self.assertEqual(
            'http://archive.ubuntu.com/ubuntu/pool/universe/m/meschach/meschach_1.2b-12_amd64.deb',
            packages[('meschach', '1.2b-12', 'amd64')])

    def assertSimpleTalSummary(self, packages):
        self.assertEqual(
            'http://archive.ubuntu.com/ubuntu/pool/main/s/simpletal/python2.4-simpletal_4.1-2ubuntu1_all.deb',
            packages[('python2.4-simpletal', '4.1-2ubuntu1', 'all')])

    def assertAtomixSparcURL(self, packages):
        self.assertEqual(
            'http://archive.ubuntu.com/ubuntu/pool/main/a/atomix/atomix_2.13.4-0ubuntu2_sparc.deb',
            packages[('atomix', '2.13.4-0ubuntu2', 'sparc')])
        
    def assertAtomixi386Summary(self, packages):
        self.assertEqual(
            'http://archive.ubuntu.com/ubuntu/pool/main/a/atomix/atomix_2.13.4-0ubuntu2_i386.deb',
            packages[('atomix', '2.13.4-0ubuntu2', 'i386')])

    def test_origin_annotation(self):
        # each package url should be annotated with the origin - the 
        # distro and component it was downloaded from.
        # libsamplerate0 is in both dapper and edgy and thus is a convenient
        # check for this.
        source = PackageDataSource()
        distros = {
            'dapper':{'main':['i386']},
            'edgy'  :{'main':['i386']},
            }
        packages, publication = source.get_all_packages(distros)
        self.assertEqual(set(['dapper/main', 'edgy/main']),
            publication['libsamplerate0_0.1.2-2_i386'])

    def test_get_all_package_lists(self):
        # there is a simple call 'get_all_packages' which gets the packages
        # for many distributions, components and archs at once.
        source = PackageDataSource()
        distributions = {
            'dapper': {
                'main':['sparc', 'i386'],
                'universe':['amd64'],
                },
            'breezy': {
                'restricted':['powerpc', 'i386']
                },
            }
        packages, publications = source.get_all_packages(distributions)
        self.assertAtomixSparcURL(packages)
        self.assertAtomixi386Summary(packages)
        self.assertSimpleTalSummary(packages)
        self.assertMeschachSummary(packages)
        self.assertLRMSummary(packages)

        # and we can do it with state in the object 
        # here we limit the default which is huge to the data we are testing
        # with
        source.set_distributions(distributions)
        packages, publications = source.get_all_packages()
        self.assertAtomixSparcURL(packages)
        self.assertAtomixi386Summary(packages)
        self.assertSimpleTalSummary(packages)
        self.assertMeschachSummary(packages)
        self.assertLRMSummary(packages)



# TODO list
# test that the manually supplied mirror overrides the defaults.
