# Copyright (C) 2006 Canonical Limited.
#
# Licence to be determined
#

"""Tests for package relationship operations."""

from unittest import TestCase

from findconflicts import (
    DebRelationship,
    )


class TestRelationshipEvaluation(TestCase):
    """Tests for the DebRelationship match() logic."""

    def test_simple_replaces(self):
        b_2_1 = ('b', '2-1', 'i386')
        # an empty replaces does not match.
        self.assertFalse(DebRelationship('').matches('b', '2-1'))
        # a replaces on a different package does not replace.
        self.assertFalse(DebRelationship('c').matches('b', '2-1'))
        # replaces on the package name match
        self.assertTrue(DebRelationship('b').matches('b', '2-1'))
        # a provides clause matches when the package name does not.
        self.assertTrue(DebRelationship('mta').matches('b', '2-1', 'mta'))
        self.assertTrue(DebRelationship('mta').matches('b', '2-1', 'foo, mta'))
        self.assertFalse(DebRelationship('mta').matches('b', '2-1', 'bar, foo'))

    def test_versioned_replaces(self):
        # a versioned replaces can be one of 5 operators:
        # <<, <=, =, >= and >> 
        b_2_1 = ('b', '2-1')
        # test each of the operators for true and false conditions:
        # strictly less
        self.assertFalse(DebRelationship('b (<< 2-1)').matches(*b_2_1))
        self.assertTrue(DebRelationship('b (<< 2-1.1)').matches(*b_2_1))
        # less or equal
        self.assertFalse(DebRelationship('b (<= 2-0)').matches(*b_2_1))
        self.assertTrue(DebRelationship('b (<= 2-1)').matches(*b_2_1))
        # and its compatability version
        self.assertFalse(DebRelationship('b (< 2-0)').matches(*b_2_1))
        self.assertTrue(DebRelationship('b (< 2-1)').matches(*b_2_1))
        # equal
        self.assertFalse(DebRelationship('b (= 2-0)').matches(*b_2_1))
        self.assertTrue(DebRelationship('b (= 2-1)').matches(*b_2_1))
        self.assertFalse(DebRelationship('b (= 2-1.1)').matches(*b_2_1))
        # strictly greater
        self.assertFalse(DebRelationship('b (>> 2-1)').matches(*b_2_1))
        self.assertTrue(DebRelationship('b (>> 2-0)').matches(*b_2_1))
        # greater or equal
        self.assertFalse(DebRelationship('b (>= 2-1.1)').matches(*b_2_1))
        self.assertTrue(DebRelationship('b (>= 2-1)').matches(*b_2_1))
        # and its compatability version
        self.assertFalse(DebRelationship('b (> 2-1.1)').matches(*b_2_1))
        self.assertTrue(DebRelationship('b (> 2-1)').matches(*b_2_1))


# TODO
# if the relationship engine is used for other than conflicts or replaces
# test it with ands not just ors,
